;************************************************************
; Touch-Memory DS1991 Ansteuerung mit Atmel AT2313 Controller                      
;                                              
; Version 1.0                                   
;                                               
; AT90S2313 mit 4 MHz Takt
;
; Schaltplan:
;
;             # +5V             
;             |                  Fehler
;             |          ____    LED
;             +---------|____|---->|----# PB2 AT902313
;             |          220 Ohm
;             -
;            | |
;  IB        |_| 5K
; Holder      |             
;  __				  |
;    |        |         Data
;  |----------+---+--------------------# PB1 AT902313
;  __|---+ 			  |
;	GND		 |			  |
;				 |	 	    |
;				 |    E   \      ____
;	       |      B  |----|____|---------# PB0 AT902313
;				 |    C   /		   5K
;        |       |   
; 			 |		   |   BC 327 (PNP)
;        |       |
;        |       |
;       ---     ---
;        -       -  GND
;
;             
;************************************************************   

.include"C:\App\AvrAsm\APPNOTES\2313def.inc"
                                                                                               
;Konstanten fuer Touch-Memory Interface         
.equ    TM_OUT    = PB0   ; PB0 sendet zum TM
.equ    TM_IN     = PB1   ; PB1 empfaengt vom TM
.equ    LED       = PB2   ; PB2 LED fuer Fehler
.equ    LED_OK    = PD6   ; PD6 LED fuer Init TM ok

                                                
;Touch-Memory Befehle                           
.equ    RD_ROM        = 0x33
.equ    SKIP_ROM      = 0xCC
.equ    SEARCH_ROM    = 0xF0

.equ    WR_SCRATCH    = 0x96 ; write ScrachPad command
.equ    RD_SCRATCH    = 0x69 ; read ScrachPad command
.equ    COPY_SCRATCH  = 0x3C ; copy from the ScrachPad
.equ 		SEL_SCRATCH		= 0xC0 ; ScrachPad Adresse 0x00 selektieren

.equ    RD_MEMORY     = 0xF0 ; fr spter
                                                
;Variablen                                      
.def    delay_cntr  = r16 ; Schleifenzaehler
.def    buffer      = r17 ; Datenbyte
.def    data_cntr   = r18 ; Datenbitzeiger
.def    hilf        = r19 ; Hilfsregister
.def    off_es      = r20 ; End Adresse
.def    high_adr    = r21 ; 2. Adress-Byte
.def    low_adr     = r22 ; 1. Adress-Byte
.def    data1       = r23 ; Datenbyte
.def    temp        = r24 ; temporaeres Register
                                                
                                                
.CSEG
.ORG    0x00      ; Programm beginnt bei 0

    
rjmp  main    ; Starte Hauptprogramm

          
;**************************************************************   
; Subroutine init
; Initialisiere Stack-Pointer, Port und Timer0. Loesche LED.
;**************************************************************   

init:   
		sbi PORTB,TM_OUT
    
    ldi temp,0b11111101   ; PB0 und PB2 Ausgang, PB1 Eingang
    out DDRB,temp
    
    ldi temp,0b01000000   ; PD6 OK LED
    out DDRD,temp
    
    ldi temp,0b00000010   ; Timer0-Vorteiler CK/8
    out TCCR0,temp
    
    sbi PORTB,LED   			; Loesche LED
    sbi PORTD,LED_OK 			; Loesche OK LED
    ret

;**************************************************************   
; Subroutine init_tm                            
; Initialisiere Touch-Memory und teste auf "Presence Pulse"   
; Uebergibt T-Flag=1, falls TM da                
;           T-Flag=0, falls TM nicht da          
;**************************************************************   

init_tm:  
    clr delay_cntr          ; Timer0 initialisieren
    out TCNT0,delay_cntr
    cbi PORTB,TM_OUT        ; Bus auf low
		
	
reset_delay:  
    in  temp,TCNT0          ; halte Bus min. 480us
    cpi temp,245            ; auf low
    brlo  reset_delay
    sbi PORTB,TM_OUT        ; Datenleitung loslassen
    ldi delay_cntr,20       ; warte min. 15 us

wait_delay: 
    dec delay_cntr          ; damit TM Bus anzieht
    brne  wait_delay                                       
    set                     ; nehme an, TM ist da
    ldi hilf,5              ; teste 5mal, ob TM da 

test_tm:  
  sbis  PINB,TM_IN          ; falls TM da, springe raus
  rjmp  tm_ok               ; 
  
  ldi delay_cntr,20         ; warte min. 15 us

wait_delay2:  
    dec delay_cntr          ; und teste nochmal
    brne  wait_delay2
    dec hilf 
    brne  test_tm           ; teste 5 mal, ob TM da
    clt                     ; TM doch nicht da! Loesche CARRY    

tm_ok:          
    clr delay_cntr          ; Schleife fuer 480 us
    out TCNT0,delay_cntr    ; zur Erholung des TMs
    cbi PORTD,LED_OK   			; OK LED einschalten  

release:  
    in  temp,TCNT0          ; 
    cpi temp,245            ; 
    brlo release
    sbis  PINB,TM_IN	      ; teste nach 480us Bus low?
    clt                     ; Bus permanent auf low, Fehler!
    ret

                                                
                                                
;***************************************************************   
; Subroutine cmd_out                            
; Subroutine cmd_out sendet Befehl bestehend aus 8 Bit zum   
; Touch-Memory. Es wird ueber Pin PB0 zum Touch-Memory   
; gesendet und ueber Pin PB1 vom Touch-Memory gelesen.   
; Command muss im Register "buffer" uebergeben werden.   
; Benutzt "data_cntr", "buffer", "delay_cntr"   
;***************************************************************   
                                                
cmd_out:  
    ldi data_cntr,8       ; Command ist 8 Bit

cmd_bit:        
    cbi PORTB,TM_OUT      ; Startzeichen
    ldi delay_cntr,3      ; halte Bus 2 us auf low

cmd_delay1:     
    dec delay_cntr
    brne  cmd_delay1                             
    ror buffer            ; schiebe Bit ins CARRY
    brcc  wr_zero         ; Ist Bit "0"?
    sbi PORTB,TM_OUT      ; Nein. Bit war eine "1"

wr_zero:        
    ldi delay_cntr,80     ; halte Bus fuer 60 us

cmd_delay:  
    dec delay_cntr
    brne cmd_delay
    sbi PORTB,TM_OUT    ; lasse nach 60 us los
    nop                             
    nop                             
    dec data_cntr       ; wiederhole alle 8 Bit
    brne cmd_bit
    nop                             
    nop                             
    ret
                                                
                                                
;***************************************************************   
; Subroutine data_in                            
; Subroutine data_in liest 8 Bit vom Touch-Memory ueber Pin PB1   
; ein.                                          
; Ueber Pin PB0 wird zum Touch-Memory gesendet. Das eingelesene   
; Byte steht in "buffer".                       
; Benutzt "buffer", "delay_cntr", "data_cntr"   
;****************************************************************   
                                                
data_in:  
  ldi data_cntr,8         ; 8 Bit aus TM lesen

read_bit: 
    cbi PORTB,TM_OUT      ; Startzeichen
    nop                   ; warte 1 us
    nop
    nop
    nop
    sbi PORTB,TM_OUT      ; lasse Bus los
    ldi delay_cntr,8  		; warte ca. 6 us

in_delay: 
    dec delay_cntr
    brne in_delay
    sec                   ; nehme an, habe "1" gelesen
    sbis PINB,TM_IN
    clc                   ; nein, war eine "0"
    ror buffer            ; schiebe Bit in "buffer"
    ldi delay_cntr,30  		; warte ca. 15 us

rd_rel_delay: 
    dec delay_cntr
    brne rd_rel_delay
    dec data_cntr
    brne read_bit
    ret


;**************************************************************   
; Subroutine fehler                             
; Sollte ein Fehler auftreten oder kein Touch-Memory   
; angeschlossen sein, so wird das ueber die LED angezeigt.   
;***************************************************************   
                                                
fehler:   
    cbi PORTB,LED   	; LED einschalten
    sbi PORTD,LED_OK  ; OK LED aus
    rjmp  fehler    	; Endlosschleife

                                                
                                                
;****************************************************************   
; Hauptprogramm                                 
; Schreibt 1 Byte ins Touch-Memory uns liest es wieder aus.   
; Start-Adresse muss in "low_adr" und "high_adr" uebergeben   
; werden. Das Byte, das geschrieben werden soll, muss im   
; W-register uebergeben werden.                 
; Das gelesene Byte steht in "buffer".          
; Sollte ein fehler auftreten oder kein Touch-Memory   
; angeschlossenen sein, wird die LED eingeschaltet.   
;****************************************************************   
                                                
main:   
    ldi temp,RAMEND         ; setze Stack-Pointer
    out SPL,temp            ; an das SRAM-Ende

    rcall init              ; initialisieren
       
                                                
    ldi low_adr,SEL_SCRATCH ; selektiere Scratchpad Adresse 0x00
    ldi data1,0x4D 					; 1Byte 4D an Adresse 0x00 schreiben              
                                        
    rcall init_tm           ; ist TM da? 
    brtc  fehler            ; ja, springe weiter ; nein -> Fehler!
                                    
;-------------------------------------------------------------------------
; Hier beginnt das Schreiben des Touch/Memory
;-------------------------------------------------------------------------
    ldi buffer,SKIP_ROM     ; ueberspringe ROM 
    rcall cmd_out           ; sende skip_rom command 
    
    ;****************************************************************
    ; 1Byte 0x96,  2.Byte 0xC0, 3.Byte Einerkomplement vom 2.Byte
    ;****************************************************************
    ldi buffer,WR_SCRATCH   ; bereite wr_scratch vor 
    rcall cmd_out           ; 1.Byte
    
    ldi buffer,SEL_SCRATCH  ; 
    rcall cmd_out           ; 2.Byte
    
    com buffer							; Einerkomplement vom 2.Byte
    rcall cmd_out           ; 3.Byte
    
    ;ldi temp, SEL_SCRATCH		; sichere byte 
    
    rcall init_tm           ; Uebertragung beenden / Reset 
          
 
   
;-------------------------------------------------------------------------
; Hier beginnt der Datenvergleich
;-------------------------------------------------------------------------
    ldi buffer,SKIP_ROM     ; ueberspringe ROM 
    rcall cmd_out                  
    
    ldi buffer,RD_SCRATCH   ; lies Scratchpad 
    rcall cmd_out                  
    
    ldi buffer,SEL_SCRATCH  ; Startadresse (0xC0) senden 
    rcall cmd_out           ; Adr zum TM uebertragen 
    
    com buffer			 				; Einerkomplement von SEL_SCRATCH 
    rcall cmd_out           ; Adr zum TM uebertragen 
    
    rcall data_in           ; Byte lesen
    cp buffer, low_adr  		; Byte vergleichen 
    brne  fehler            ; Vergleich negativ, Fehler! 
        
    mov off_es,buffer       ; OFFSET/ES sichern
                                              
; Nach Datenvergleich werden die Daten aus dem
; Scratchpad in die endgueltige Speicherstelle

; kopiert
    rcall init_tm                  
    ldi buffer,SKIP_ROM
    rcall cmd_out                  
    
    ldi buffer,COPY_SCRATCH
    rcall cmd_out                  
    
    mov buffer,low_adr                
    rcall cmd_out                  
              
    mov buffer,off_es                 
    rcall cmd_out                  
    
    rcall init_tm                  
                                                                       
    out PORTB,buffer
                                                
loop:   rjmp  loop        ; Endlosschleife 
                                                
                                                


